@@ -1,5 +1,4 @@
 <?php
-
 /**
  * User by ID action class.
  *
@@ -47,17 +46,17 @@ class UserbyidAction extends Action
 {
      /**
      * Is read only?
-     * 
+     *
      * @return boolean true
      */
     function isReadOnly($args)
-    {                
+    {
         return true;
     }
 
      /**
      * Class handler.
-     * 
+     *
      * @param array $args array of arguments
      *
      * @return nothing
@@ -67,10 +66,12 @@ class UserbyidAction extends Action
         parent::handle($args);
         $id = $this->trimmed('id');
         if (!$id) {
-            $this->clientError(_('No id.'));
+            // TRANS: Client error displayed trying to find a user by ID without providing an ID.
+            $this->clientError(_('No ID.'));
         }
-        $user =& User::staticGet($id);
+        $user = User::staticGet($id);
         if (!$user) {
+            // TRANS: Client error displayed trying to find a user by ID for a non-existing ID.
             $this->clientError(_('No such user.'));
         }
 
@@ -88,4 +89,3 @@ class UserbyidAction extends Action
         common_redirect($url, 303);
     }
 }
-
