@@ -172,8 +172,8 @@ class TwitterImport
 
         $notice->is_local   = Notice::GATEWAY;
 
-        $notice->content  = html_entity_decode($status->text, ENT_QUOTES, 'UTF-8');
-        $notice->rendered = $this->linkify($status);
+        $notice->content  = html_entity_decode($this->linkify($status, FALSE), ENT_QUOTES, 'UTF-8');
+        $notice->rendered = $this->linkify($status, TRUE);
 
         if (Event::handle('StartNoticeSave', array(&$notice))) {
 
@@ -339,10 +339,7 @@ class TwitterImport
     {
         global $config;
 
-        $path_parts = pathinfo($twitter_user->profile_image_url);
-
-        $newname = 'Twitter_' . $twitter_user->id . '_' .
-            $path_parts['basename'];
+        $newname = 'Twitter_' . $twitter_user->id . '_' . basename($twitter_user->profile_image_url);
 
         $oldname = $profile->getAvatar(48)->filename;
 
@@ -370,15 +367,15 @@ class TwitterImport
 
         $path_parts = pathinfo($twitter_user->profile_image_url);
 
-        $img_root = substr($path_parts['basename'], 0, -11);
-        $ext = $path_parts['extension'];
-        $mediatype = $this->getMediatype($ext);
+        $ext = (isset($path_parts['extension']) ? '.'.$path_parts['extension'] : '');	// some lack extension
+        $img_root = basename($path_parts['basename'], '_normal'.$ext);	// cut off extension
+        $mediatype = $this->getMediatype(substr($ext, 1));
 
         foreach (array('mini', 'normal', 'bigger') as $size) {
             $url = $path_parts['dirname'] . '/' .
-                $img_root . '_' . $size . ".$ext";
+                $img_root . '_' . $size . $ext;
             $filename = 'Twitter_' . $twitter_user->id . '_' .
-                $img_root . "_$size.$ext";
+                $img_root . '_' . $size . $ext;
 
             $this->updateAvatar($profile->id, $size, $mediatype, $filename);
             $this->fetchAvatar($url, $filename);
@@ -401,8 +398,9 @@ class TwitterImport
         $mediatype = null;
 
         switch (strtolower($ext)) {
+        case 'jpeg':
         case 'jpg':
-            $mediatype = 'image/jpg';
+            $mediatype = 'image/jpeg';
             break;
         case 'gif':
             $mediatype = 'image/gif';
@@ -419,16 +417,15 @@ class TwitterImport
         global $config;
 
         $path_parts = pathinfo($user->profile_image_url);
-        $ext = $path_parts['extension'];
-        $end = strlen('_normal' . $ext);
-        $img_root = substr($path_parts['basename'], 0, -($end+1));
-        $mediatype = $this->getMediatype($ext);
+        $ext = (isset($path_parts['extension']) ? '.'.$path_parts['extension'] : '');
+        $img_root = basename($path_parts['basename'], '_normal'.$ext);
+        $mediatype = $this->getMediatype(substr($ext, 1));
 
         foreach (array('mini', 'normal', 'bigger') as $size) {
             $url = $path_parts['dirname'] . '/' .
-                $img_root . '_' . $size . ".$ext";
+                $img_root . '_' . $size . $ext;
             $filename = 'Twitter_' . $user->id . '_' .
-                $img_root . "_$size.$ext";
+                $img_root . '_' . $size . $ext;
 
             if ($this->fetchAvatar($url, $filename)) {
                 $this->newAvatar($id, $size, $mediatype, $filename);
@@ -540,7 +537,7 @@ class TwitterImport
     const HASHTAG = 2;
     const MENTION = 3;
 
-    function linkify($status)
+    function linkify($status, $html = FALSE)
     {
         $text = $status->text;
 
@@ -596,13 +593,21 @@ class TwitterImport
             $orig = $this->twitEscape(mb_substr($text, $start, $end - $start));
             switch($type) {
             case self::URL:
-                $linkText = $this->makeUrlLink($object, $orig);
+                $linkText = $this->makeUrlLink($object, $orig, $html);
                 break;
             case self::HASHTAG:
-                $linkText = $this->makeHashtagLink($object, $orig);
+                if ($html) {
+                    $linkText = $this->makeHashtagLink($object, $orig);
+                }else{
+                    $linkText = $orig;
+                }
                 break;
             case self::MENTION:
-                $linkText = $this->makeMentionLink($object, $orig);
+                if ($html) {
+                    $linkText = $this->makeMentionLink($object, $orig);
+                }else{
+                    $linkText = $orig;
+                }
                 break;
             default:
                 $linkText = $orig;
@@ -630,9 +635,13 @@ class TwitterImport
         return htmlspecialchars(html_entity_decode($str, ENT_COMPAT, 'UTF-8'));
     }
 
-    function makeUrlLink($object, $orig)
+    function makeUrlLink($object, $orig, $html)
     {
-        return "<a href='{$object->url}' class='extlink'>{$orig}</a>";
+        if ($html) {
+            return '<a href="'.htmlspecialchars($object->expanded_url).'" class="extlink">'.htmlspecialchars($object->display_url).'</a>';
+        }else{
+            return htmlspecialchars($object->expanded_url);
+        }
     }
 
     function makeHashtagLink($object, $orig)
